"""Machine learning using crowdsourced labels on the Radio Galaxy Zoo.

MIT-licensed. See license.txt.

Matthew Alger & Cheng Soon Ong
The Australian National University
2016
"""

__version__ = '0.0.1a1'
__description__ = 'Automated cross-identification of radio objects and host ' \
                  'galaxies using crowdsourced labels from the Radio Galaxy ' \
                  'Zoo.'
